# AmiTestHarness.pl
#
# AmiBug.Com
# Perl Based Test Harness Frame
#
# Author: Robert Sabourin
# Copyright 2003 AmiBug.Com, Inc.
#
# Revision History:
#
# 2003-07-31
# Created original perl framework based on Rational Robot and Visual Basic versions
#
# 2003-08-11
# Added hl7 aware file comparison
# Added command line parameter for input and output directory
#
#
# Documentation:
#
#  perl AmiTestHarness.pl inpath outpath
#
#  inpath is source directory
#
#  outpath is directory which will be created for output files and test results
#
#  Test Harness parses comma seperated value file Manifest.csv from source directory.
#
#  Syntax of Manifest.csv is such that each row includes in column 1 a test id,
#  in column 2 a test verb and in other columns various parameters dependent on the
#  specific verb.  Detailed syntax is descibed in the syntax worksheet of 
#  Manifest.xls.
#
#  All files references in Manifest.csv are relative to the directory in which 
#  Manifest.csv is found.
#
#  Parsing continues line by line until either end of file or an END directive is
#  encountered.
#
#  Test resulting are placed in a new subdirectory, under that in which Manifest.csv
#  is found.  The directory is created with each test run (invocation of
#  AmiTestHarness.pl) and is named by a string on digits representing the 
#  YEAR_MONTH_DAY_HOUR_MINUTE_SECOND of the test run.
#
#  Test results include:
#  - a log file for the test run (csv format) 
#      includes pass fail status
#  - a log file for each test case (row of manifest.csv) (csv format)
#      includes pass fail status
#  - a result file for verbs which return a result as per syntax.
#
# Start of code!
#
# We will be using higher resolution time functions for test case time stamping
#
use Time::HiRes qw( usleep ualarm gettimeofday tv_interval );
#
# text parsing to decode csv files!
#
use Text::ParseWords;
#
# switch statement
#
use Switch;
#
# Split into an array of parameters
#
use Text::ParseWords;
#
# Get File::Copy object to manipulate files
#
use File::Copy;
#
# Get File::Compare object to compare files
#
use File::Compare;
#
# IO via socket and tcpip
#
use IO::Socket;
#
# Polling Sockets
#
use IO::Select;

$inpstring = $ARGV[0];   # get input path from parameter

$outstring = $ARGV[1];  # get output path from parameter

if ( $inpstring ne "" ) {

    chdir $inpstring;   # make it the active directory

}

if ( $outstring eq "" ) {
    #
    # If output path is undefined then make it out of current time to second
    #
    # Get current time to second from time functions
    #
    ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = gmtime(time);
    #
    # Name of output directory is built from time to second formated with sprintf
    #
    $outstring = sprintf( "%04s_%02s_%02s_%02s_%02s_%02s", $year+1900, $mon+1, $mday, $hour, $min, $sec);
    #
    # Create test result directory
    #
}

mkdir $outstring, 777;
#
# Make test result directory the active directory for test execution
#
chdir $outstring;
#
# Place test script source in result directory
#
copy("..\\AmiTestHarness.pl",".");
#
# Place matching test manifest in result directory
#
copy("..\\manifest.csv",".");
#
# Open Test Manifest for reading
#
$testmanifestfile="manifest.csv";
open(TESTMANIFEST,'<',$testmanifestfile);
#
# Now cycle through the Test Manifest
#
$inputstring=<TESTMANIFEST>;
#
# @new[0] is first parameter
# @new[1] is second parameter
# etc
#
    @new = quotewords(",", 0, $inputstring);
#
# Now loop
#
#
# Initialize Test Output Counter to zero
#
$tocount = 0;
while (@new[1] ne "END") {
    #
    # Case statement based on the verb from parameter @new[1]
    #
    $verb = @new[1];
    $testname = @new[0];
    $logname = join(".", $testname, "csv");
    open(TESTCASELOG,'>',$logname);
    #
    # Syntax described in Manifest.xls
    #
    #
    # Get current time to second from time functions at start of verb execution
    #
    ($stsec, $stmin, $sthour, $stmday, $stmon, $styear, $stwday, $styday, $stisdst) = gmtime(time);
    #
    # get seconds and microseconds since the epoch
    #
    ($sts, $stusec) = gettimeofday;
    #
    # execute command as indicated by verb
    #
    switch($verb) {
        #
        case('START'){
        $logstring = join ",", "Start String", $testname, $verb;
        }
        #

        case('END'){
        $logstring = join ",", "End String", $testname, $verb;
        }

        case('DELAY'){
        $logstring = join ",", "End String", $testname, $verb;
        #
        # Delay for n seconds
        #
        # Parameters
        #
        # @new[0] Test Name
        # @new[1] Verb 
        # @new[2] Delay in Seconds
        # @new[3] NA
        # @new[4] NA
        # @new[5] NA
        # @new[6] NA
        # @new[7] NA
        # @new[8] NA
        #
        # Grab Server Address and Port Number
        #
        $del = @new[2];
        sleep $del;
        }
        #
        #
        #

        case('HOPEN'){
        $logstring = join ",", "Hopen String", $testname, $verb;
        #
        # Open Socket
        #
        # Parameters
        #
        # @new[0] Test Name
        # @new[1] Verb 
        # @new[2] Server Address
        # @new[3] Port Number
        # @new[4] NA
        # @new[5] NA
        # @new[6] NA
        # @new[7] NA
        # @new[8] NA
        #
        # Grab Server Address and Port Number
        #
        $host = @new[2];
        $port = @new[3];
        #
        # create a tcp connection to the specified host and port
        #
        $hl_handle = IO::Socket::INET->new(Proto     => "tcp",
                                           PeerAddr  => $host,
                                           PeerPort  => $port);
        $hl_handle->autoflush(1); # so output gets there right away
        # End of case HOPEN
        #
        # IOSELECT for polling whether data is ready for reading
        #
        $readable = IO::Select->new;         # Create a new IO::Select object
        $readable->add($hl_handle);          # Add our tcp connection (handle) to it
        }
        #

        case('HSEND'){
        #
        # Parameters
        #
        # @new[0] Test Name
        # @new[1] Verb 
        # @new[2] Source File Name
        # @new[3] Output File Name
        # @new[4] Delay in seconds before sending message
        # @new[5] Master File Name
        # @new[6] Timeout in seconds max time waiting for and ACK
        # @new[7] NA
        # @new[8] NA
      
        $logstring = join ",", "Hsend String", $testname, $verb, @new[2];

        #
        # grab delay
        #
        $before_delay = @new[4];
        #
        # sleep for delay before send
        #
        sleep($before_delay);
        #
        # Copy input file to test directory
        #
        $infile = @new[2];
        chomp $infile;
        copy("..\\" . $infile,".");
        #
        # Open $infile and read it into buffer BIN
        #
        open(INFILEA, "<", $infile);
        @BIN=<INFILEA>;
        close(INFILEA);
        #
        $readtimeout=@new[6]; # max wait for a response in seconds
        chomp $readtimeout;
        #
        # Send message to socket
        #
        @$ready = $readable->can_write($readimeout);
        $mes="FAIL";
        foreach my $t (@$ready) {
                $mes = "PASS"; # no timeout failure!
                #
                # $t is handle to print to
                #
                # body from infile
                #
                print $t @BIN;
                #
                # Magic determined by experimental debugging - if not included then the
                #
                print $t chr(0x0b), chr(0x1C), chr(0x0D); 
                #                
                # last HSEND will not be complete - due to file format of messages!
                #
            }

        #
        # Now read socket to get ACKNOWLEDGEMENT
        #


 
        $outfile = @new[3];   # Output file name TestName.out
        chomp $outfile;
        open(OUTFILA, ">", $outfile);    

        #
        # Is there any data ready to read?
        #
        @$ready = $readable->can_read($readtimeout);


        $mes="FAIL"; # assume read timeout failure


            #
            # Read Response ACK one byte at a time
            #
            foreach my $s (@$ready) {
                $mes = "PASS"; # no timeout failure!
                #
                # $s is handle with input to read!
                #
                $bytes_read = recv($s, $instr, 1, 0);


                while (ord($instr) != 0x1C){
                    print OUTFILA $instr;
                    
                    $res = recv($s, $instr, 1, 0);

                }
                print OUTFILA $instr;
            }

        close(OUTFILA);

        if ($mes eq "FAIL"){
            #
            # Failure due to timeout on read
            #
            $test_result = "FAIL_TIMEOUT";
            
        } else {
            # 
            # check if test cases passes or fails by comparing actual with expected results
            #
            # (1) Copy master file to test directory
            #
            $masterfile=@new[5];
            chomp $masterfile;
            copy("..\\" . $masterfile,".");
            #
            # (2) compare output file to master file
            #
            $test_result="PASS"; # assume test passes
            #
            $mask = "T|T|T|T|T|T|X|T|T|T|T|T|T|T|T|T|T|T|T|T|T|T|T|T|"; # mask for comparison only compare
                                                                        # Hl7 fields with T
            #
            if (&AmiHl7Cmp($masterfile,$outfile,$mask)){
                #
                # Test Passes
                #
                $test_result = "PASS";
            } else {
                #
                # Test Fails
                #
                $test_result = "FAIL";
            }
         }

        $temstr = $logstring;
        $logstring = join ",", $temstr, $test_result;

        #
        # *** IF FAIL THEN CLOSE AND REOPEN SOCKET ***
        # *** NOTE SERVER CLOSES CONNECTION ON SOME FAILURES ***
        #
        if ( $test_result ne "PASS" ) {
            #
            # stop reading from previous hl_handle
            #
            $readable->remove($hl_handle); # remove old handle from ioselect list
            #
            # Close handle
            #
            close($hl_handle);             # close old handle once and for all
            #
            # reopen using exact code from HOPEN above
            #
            # create a new tcp connection to the specified host and port (from prev hopen)
            #
            $hl_handle = IO::Socket::INET->new(Proto     => "tcp",
                                               PeerAddr  => $host,
                                               PeerPort  => $port);
            $hl_handle->autoflush(1); # so output gets there right away
            #
            # IOSELECT for polling whether data is ready for reading
            #
            $readable->add($hl_handle);          # Add our tcp connection (handle) to list
            } # end if not pass
        #
        } # end case of hsend
 
        case('HCLOSE'){
        $logstring = join ",", "Hclose String", $testname, $verb;
        #
        # Close the socket
        #
        close($hl_handle);
        }
        #
        else{
        $logstring = join ",", "Helse String", $testname, $verb;
        }
    }
    #
    # get seconds and microseconds since the epoch
    #
    ($ens, $enusec) = gettimeofday;
    #
    # Get current time to second from time functions at end of verb execution
    #
    $tem = join ",", $sts, $stusec, $ens, $enusec, $logstring;
    $logstring = $tem;
    #
    # Log for test case
    #
    print TESTCASELOG $logstring, "\n";
    #
    # Close test case log file
    #
    close(TESTCASELOG);
    #
    # Log for test run
    #
    $testrunlogfile="Logfile.csv";
    open(TESTRUNLOG,'>>',$testrunlogfile);
    print TESTRUNLOG $logstring, "\n";
    close(TESTRUNLOG);
#
#    DEBUGprint $logstring, "\n";
#    #
    #
    # Get next line of text
    #
    $inputstring=<TESTMANIFEST>;
    #
    # Split into parameter array
    #
    @new = quotewords(",", 0, $inputstring);
    #
    # continue while loop
    #
}
#
# clean up
#
# close TESTMANIFEST
#
close(TESTMANIFEST);
#


########################################################################################################
########################################################################################################
########################################################################################################
########################################################################################################


sub AmiHl7Cmp {
#
# compare file p1 with p2 based on mask of p3
# mask is T or other seperated by | similar to HL7 format
#
my $scrnm = @_[0];    # source file name
my $mstnm = @_[1];    # master file name
my $mskstr = @_[2];   # comparison mask - T indicates compare

my $temstr = "";      # temporary string variable
#
# read source file
#
open(INFILEA, "<", $scrnm);
my $srcstr=<INFILEA>;
close(INFILEA);

$temstr = &AmiStrip( $srcstr );    # strip control characters from source file
$srcstr = $temstr;
#
# read master file
#
open(INFILEA, "<", $mstnm);
my $mststr=<INFILEA>;
close(INFILEA);

$temstr = &AmiStrip( $mststr );    # strip control characters from master file
$mststr = $temstr;


$len_mst = length($mststr);        # length of master
$len_msk = length($mskstr);        # length of mask

my @src = split(/\|/,$srcstr); # split HL7 source into an array

my @mst = split(/\|/,$mststr); # split HL7 master into an array

my @msk = split(/\|/,$mskstr); # split HL7 mask into an array


my $max = @src;                    # number of elements in mask
#
# DEBUGprint "MAX ", $max , "\n";
#

my $i = 0;
my $OK = "TRUE";

while (( $i <= $max) & ($OK eq "TRUE")){
    if ($msk[$i] ne "X"){
        if ($src[$i] ne $mst[$i]){
            $OK="FALSE";
            }
        }
        $i = $i + 1; # next $i
    }

if ( $OK eq "TRUE" ){
    return(1);
    } else {
    return(0);
}
} # end of sub AmiHl7Cmp


########################################################################################################
########################################################################################################
########################################################################################################
########################################################################################################


sub AmiStrip {
#
# input is a string
# output is same string with all control characters stripped out
#
my $inputstring = @_[0]; # string to stip from parameter

my $outputstring = "";   # stripped string start as empty

my $fred = "";           # temporary character placeholder initially empty

my $tem = "";            # temporary outputstring placeholder initially empty

my $i=0; # counter position on input

my $len = length($inputstring); # get number of characters of input

while ($i < $len){
    $fred = substr($inputstring,$i,1);
    if ( ( ord($fred) > 0x1F ) && ( ord($fred) < 0x7F ) ) {
        $tem = $outputstring . $fred;
        $outputstring = $tem;
    }
$i = $i + 1;
}

return($outputstring);
} # end of subroutine AmiStrip